#include "Isotronic.h"

// states of reception
#define S_WAIT    1     // wait for the >60ms low at the beginning of a transmission
#define S_COLLECT 2     // wait for end of low pulse containig data, long pulse is 0, short 1
#define S_READY   3     // 12 data pulses have been collected, result is usable
#define S_HIGHSHORT 4   // short high pulse after long low pulse
#define S_HIGHLONG  5   // long high pulse after short low pulse

// for interal timing function micros is used
// to reduce the number of long calculations the
// result of micros is divided by T_US for further
// calculations
#define T_US      64
#define T_60MS    (60000/T_US)
#define T_0MS5    (500/T_US)
#define T_0MS7    (700/T_US)
#define T_0MS8    (800/T_US)
#define T_0MS9    (900/T_US)
#define T_1MS0    (1000/T_US)
#define T_1MS5    (1500/T_US)
#define T_2MS2    (2200/T_US)

isotronic::isotronic( uint8_t pin ) {
  _pin = pin;
  _state = S_WAIT;
}

isotronic::~isotronic() {
}

uint8_t isotronic::getKey() {
  return _key;
}

uint16_t isotronic::getAddress() {
  return _addr;
}

// debug
uint16_t isotronic::getLast() {
  return _last;
}

// debug
uint8_t isotronic::getState() {
  return _state;
}

uint8_t isotronic::available() {
  return ( _state == S_READY );
}

void isotronic::release() {
  _state = S_WAIT;
}


const char* isotronic::getKeyText ( uint8_t key ) {
  switch ( key ) {
    case ISO_1ON: return "1 ON";
    case ISO_1OFF: return "1 OFF";
    case ISO_1UP: return "1 UP";
    case ISO_1DOWN: return "1 DOWN";
    
    case ISO_2ON: return "2 ON";
    case ISO_2OFF: return "2 OFF";
    case ISO_2UP: return "2 UP";
    case ISO_2DOWN: return "2 DOWN";
    
    case ISO_3ON: return "3 ON";
    case ISO_3OFF: return "3 OFF";
    case ISO_3UP: return "3 UP";
    case ISO_3DOWN: return "3 DOWN";
    
    case ISO_4ON: return "4 ON";
    case ISO_4OFF: return "4 OFF";
    case ISO_4UP: return "4 UP";
    case ISO_4DOWN: return "4 DOWN";
    
    case ISO_ALLUP: return "All UP";
    case ISO_ALLDOWN: return "All DOWN";
    case ISO_ALLON: return "All ON";
    case ISO_ALLOFF: return "All OFF";
    default: return "unknown";
  }
}

void isotronic::rxInterrupt () {
  uint16_t now = (micros()+32) >> 6;
  uint16_t high, low;
  uint8_t dat;

  switch ( _state ) {
    case S_WAIT:
      if ( ( (now - _last) > T_60MS ) && digitalRead ( _pin ) ) {
        _state = S_HIGHSHORT;
        _idx = 0;
        _key = 0;
        _addr = 0;
      }
      break;
    case S_COLLECT: 
      low = now - _last;
      if ( (low > T_0MS5) && (low < T_0MS8) ) {
        dat = 1;
        _idx++;
        _state = S_HIGHLONG;
      } else if ( low < T_1MS5 ) {
        dat = 0;
        _idx++;
        _state = S_HIGHSHORT;
      } else  // pulse too long, stop reception
        _state = S_WAIT;

      if ( _idx > 12 ) {
        _key <<= 1;
        _key |= dat;
      } else {
        _addr <<= 1;
        _addr |= dat;
      }

      if ( _idx >= 20 ) {
        _state = S_READY;
      }
      break;
    case S_HIGHSHORT:
      high = now - _last;
      if ( (high < T_0MS5) || (high > T_0MS8) )
        _state = S_WAIT;
      else
        _state = S_COLLECT;  
      break;
    case S_HIGHLONG:
      high = now - _last;
      if ( (high < T_1MS0) || (high > T_1MS5) )
        _state = S_WAIT;
      else
        _state = S_COLLECT;  
      break;
    case S_READY:
      break;
    default:
      _state = S_WAIT;      
  }
  _last = now;
}


